 /***************************************************************
 *                                                              *  
 *    o_surface.h                                               *
 *    Virtual surface (gxf buffer)                              *
 *                                                              *
 *    (G) corTeX / Optimum   1999-02-04                         *
 *    http://www.citeweb.net/optimum                            *
 *                                                              *
 *    (G) Gnu GPL                                               *
 ***************************************************************/



/*
 * Obviously 'surfaces' suffer from inconsitencies
 * * Why the need of a PHYSURF_RAM ?? (used in gfx loader... ok)
 * * Why allocate buffer into surface (for conversion) instead
 * *     of into a real dedicated [conversion]structure
 * *
 * * two surfaces of a list cannot share the same ximage
 * *
 *
 */

#ifndef O_SURFACE_H
#define O_SURFACE_H

#include "o_types.h"
#include "o_std.h"
#include "o_gfxm.h"
#include "o_xutils.h"
#include "o_convert.h"


// codes for the physical display method

#define OPL_PHYSURF_NONE   (-1)
#define OPL_PHYSURF_RAM    0        // Internal Ram buffer only
#define OPL_PHYSURF_X      1        // for XImage


/*
 * #define OPL_PHYSURF_DGA    2
 * #define OPL_PHYSURF_SVGA   3
 * #define OPL_PHYSURF_FB     4
 *
 * One day, maybe...
 */



// palette mode/size
#define  PAL_NONE    (-1)
#define  PAL_RGB     1
#define  PAL_8       2
#define  PAL_16      3
#define  PAL_32      4


// surface list flags

// flags initialized by optilib
#define  OPL_SLIST_NOTHING       0
#define  OPL_SLIST_EXIST         1

// flags that program can set and change 
#define  OPL_SLIST_VISIBLE       2  
#define  OPL_SLIST_UPDATED       4  /* when the program has to mark surfaces as updated or unchanged */


// flags usually set at creation and not changed after
#define  OPL_SLIST_AUTOREDRAW    (1<<8)  /* Optilib can auto-reblit this surface when the XWindow get masked then
					    visible again. Else it will wait an explicit blit . 
					    (useful only for surface updated rarely )
					    XXX not implemented yet, use AUTOUPDATEEXPOSE instead
					 */
#define  OPL_SLIST_AUTODOWNDATE  (1<<9)  /* when true, automaticaly 'downdate' (clean update flag) it
					    after blitting 
					    (useful for surfaces unfrequently updated (like 1frame on 3 only)
					 */
#define  OPL_SLIST_AUTOUPDATEEXPOSE (1<<10) /* when true, the element will be marked as UPDATED
					       by ofn_ProcessSurfaceListEvents() when it detect
					       that the surface has been obscured then visible again,
					       to force a Blit next frame.
					       Used in conjunction with AUTODOWNDATE, it is a
					       workaround before AUTOREDRAW is implemented.
					    */
// flags set only by the optilib, dont touch them
#define  OPL_SLIST_WASVISIBLE (1<<16)       /* copy of the VISIBLE flag from last frame.
					       used to update things when made visible again.
					       (like update X colormap..)
					    */


// RGB color (0-255)
typedef struct{
  uint8 r,g,b;
  uint8 a;        // for explicit padding... or maybe alpha
} ot_color;

#define MODE_OT_COLOR MODE_bRGBA_8888  // for some nasty fast conversion ;)

// "physical" surface information (ie XImage...)
typedef struct{

  int gfxMode;   // physical color representation
  int phyMode;   // type of physical display method ( OPL_PHYSURF_X ,...)
  int palMode;   // if some sort of palette or lookup-table is used
                 // XXX not used


  union {   // the actual data (depending on X / dga/...)

    struct {            // OPL_PHYSURF_X
      ot_Ximage *oxi;
      ot_sub_window *osw;
    } x;

    void *RamBuffer;   // OPL_PHYSURF_RAM
  } i;

  struct {  // conversion palette if needed
    ot_color *pRGB;     // exact palette used (reference)
    /* XXX pRGB is also used for Hicolor -> 332 palette display
     * (for surfacelistBlit auto-update cmap) 
     * I guess it is a really horrible kludge...
     */

    uint8    *p8;       // lookup palette for inter-mode conversion
    uint16   *p16;      // only one these 3 will be used)
    uint32   *p32;

    void     *pAnything;     // only to free the pXxxx pointer whithout
                             // knowing each one is used (8/16/32 or NULL)
  } p;            

  
  
} ot_phySurface;


// "virtual" surface
typedef struct{
  int width, height;
  int gfxMode;        // RGB_565, Palette, etc
  void *buffer;       // pointer to the [virtual] framebuffer (linear (should be  ))

  int sameBuffer;     // boolean; true if (buffer = phySurface->data)
                      // (so dont free() twice )
  ot_phySurface *ops;
  

} ot_surface;


// surface list
typedef struct{
  int          number;    // number of surfaces in the list
                         // (size of the dynamic arrays)
  // the dynamic arrays :
  uint32       *flags;   // oring OPL_SLIST_xxx
  ot_surface  **list;

} ot_surface_list;


// simple surface, only used to hold gfx not to be displayed on screen
// (used by the gfx file loaders, for example)
typedef struct{
  int width, height;
  int gfxMode;        // RGB_565, Palette, etc
  void *buffer;   
  ot_color *pRGB;     // palette in case of MODE_PAL_8
} ot_simple_surface;

/**************************************/
/* Creation and attaching of surfaces */


// allocates the fields inside the passed simple_surface structure,
// (malloc the buffer and the pallette if needed).
void ofn_SimpleSurfaceCreate(ot_simple_surface *Os,
			     int Width, int Height,
			     int gfxMode);

// free internal buffers
void ofn_SimpleSurfaceDestroy(ot_simple_surface *Os);

// only fills some fields;
// specifically, does Not allocate buffer memory, unlike ofn_SimpleSurfaceCreate.
// (because it can be shared into an XImage in next Attach().
// To be used, you must Attach() it... even if it is AttachNone()
void ofn_SurfaceCreate( ot_surface *Os,
		      int Width, int Height,
		      int gfxMode);



// This one Does allocate memory for the buffer.
// Such surfaces are not displayable.
/* XXX deprecated, use ot_simple_surface instead */
void ofn_SurfaceAttachMemory( ot_surface *Os);



// This one create an Ximage (ot_ximage) to interface with
// the top Xwindow. If the gfxMode of the surface is the same than
// the Xdisplay, surface->buffer = ximage->buffer;
// Otherwise a new buffer is allocated in the surface,
// and conversion will take place for each screen-blitting.
// The image will be associated with a subwindow, at (x,y)
// inside the top_window.
// The surface will be added to the surface_list inside the top
// window, with these attributes flagged true:
//  * exist
//  * visible
void ofn_SurfaceAttachX( ot_surface *Os,
		       ot_top_window  *Ow, 
		       int x, int y);

// Break the links, destroy the XImage, free the local
// buffer
void ofn_SurfaceDetach( ot_surface *Os);


// free all the remaining stuff in the surface 
/* XXX does nothing... */
void ofn_SurfaceDestroy( ot_surface *Os);





/***********************/
/* Additional settings */

// AFTER attaching, if:
//  * the mode requires a palette (8bits indexed) 
//  (332 palette is set by default )
// It must be done after attaching, because datas must be
// converted to some internal formats depending of
// the physical surface. (and possibly update video palette too)
// This can be done multiple times.
// the Oc[] is copied internally and doesnt need to exist after this call.

void ofn_SurfaceSetPalette( ot_surface *Os,  ot_color Oc[]);


// sets a RGB 332 palette
void ofn_SurfaceSet332Palette( ot_surface *Os);


// pour les conversions 16->8 par exemple ?..
// void ofn_SurfaceSetLookup( ot_surface *Os,  ot_color Oc[]);


/****************/
/*   Blitting   */


// display the infamous thing on screen
// (whatever the flags are) (i.e even if 'not visible')
void ofn_SurfaceBlit( ot_surface *Os);

/*
** Display all the surfaces of the list, 
** depending of the conditionnal mask.
** For each surface in the list :
** if (surface.flag & mask)==mask then it is blitted
** if !=mask, not displayed
** (the flag.exist is always checked ;)  )
** It also clears the UPDATED flag if AUTODOWNDATE is set.
*/
void ofn_SurfaceListBlit( ot_surface_list *Osl, uint32 mask);


/*
** Make a snapshot of a surface list:
** create a ot_simple_surface of the same size as the window,
** then convert all surfaces to the same color mode (gfxMode) and put them
** (with their specified coordinates) in this ot_simple_surface.
** Returns : the ot_simple_surface.
**
** (XXX permettre le mode palette si toutes les surfaces sont PAL_8) 
** Only TrueColor modes are valid for the destination image.
** There is no checking for sub_window crossing the top_window borders,
** so such (you're stupid) cases will cause invalid output and
** probably coredumps.
*/

ot_simple_surface *ofn_SurfaceListSnapshot( ot_surface_list *Osl, int gfxMode, uint32 mask);


/***********************/
/* Surface List stuff  */



// init the fields of an empty list
void ofn_SurfaceListInit(ot_surface_list *osurfl);

// destroy the list, but not the elements
void ofn_SurfaceListDestroy(ot_surface_list *osurfl);

// destroy the list AND the listed surfaces
void ofn_SurfaceListDestroyAll(ot_surface_list *osurfl);

// add a surface to a list, with the specified flags (except .exist)
// return :
//  #  the number of this surface in the list
//  -1 if failed
int ofn_SurfaceListAddElement(ot_surface_list *osurfl,ot_surface *osurf, uint32 flags);

 
// remove the #nth surface from the list; doesnt detroy the surface
void ofn_SurfaceListRemoveElement(ot_surface_list *osurfl, int handle);

// remove the #nth surface from the list AND detroy the surface
void ofn_SurfaceListDeleteElement(ot_surface_list *osurfl, int handle);



#endif
